#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Jussi Määttä 2014

from sys import stdin
from operator import itemgetter
from math import log2, ceil

#
# Given a string, return a list of tuples of the form (symbol, freq),
# sorted in decreasing order by count.
#
def get_freqs(text):
    # Compute the symbol counts (proportional to their observed frequencies)
    counts = {}
    for symbol in text:
        if symbol in counts:
            counts[symbol] += 1
        else:
            counts[symbol] = 1

    # Convert the dictionary into a list of tuples of the form (symbol, count)
    counts = list(counts.items())

    # Sort in decreasing probability
    counts.sort(key=itemgetter(1), reverse=True)

    # Convert to frequencies and return
    c = sum([ count for (symbol, count) in counts ])
    return [ (symbol, count / c) for (symbol, count) in counts ]

#
# Given a sorted list of symbol frequencies, construct the Shannon--Fano
# code. Leave the argument 'codebook' to its default value of {}, it is
# used when the function calls itself recursively.
#
def get_codebook(freqs, codebook = {}):
    # Stopping condition
    if len(freqs) < 2:
        return codebook

    # Find the splitting point
    best_split_point = None
    best_diff = None
    for i in range(1, len(freqs)):
        p_left  = sum([ p for (symbol, p) in freqs[:i] ])
        p_right = sum([ p for (symbol, p) in freqs[i:] ])
        diff = abs(p_left - p_right)
        if (best_diff == None) or (diff < best_diff):
            best_diff = diff
            best_split_point = i

    # Update the codebook
    for i in range(len(freqs)):
        symbol = freqs[i][0]
        if symbol not in codebook:
            codebook[symbol] = ""

        if i < best_split_point:
            codebook[symbol] += "0"
        else:
            codebook[symbol] += "1"

    # Recursively process the splits
    codebook = get_codebook(freqs[:best_split_point], codebook)
    codebook = get_codebook(freqs[best_split_point:], codebook)

    return codebook

def main():
    # process the input and construct the Shannon--Fano code
    text = stdin.read()
    freqs = get_freqs(text)
    codebook = get_codebook(freqs)

    # display the codebook
    print("SYMBOL\tCODEWORD")
    print("------\t--------")
    for (symbol, _) in freqs:
        print(repr(symbol), codebook[symbol], sep='\t')
    print("")

    # compute the entropy
    entropy = 0.0
    for (_, p) in freqs:
        entropy -= p * log2(p)

    # compute the code-length
    code_length = 0.0
    for (symbol, p) in freqs:
        code_length += len(codebook[symbol]) * p

    # compute the expected code-length of the Shannon code
    shannon_e_code_length = 0.0
    for (_, p) in freqs:
        shannon_e_code_length += ceil(log2(1.0/p)) * p

    # print out the results
    print("Entropy:                                  ", entropy)
    print("Code-length:                              ", code_length)
    print("Expected code-length of the Shannon code: ", shannon_e_code_length)

###

if __name__ == "__main__":
    main()